﻿//
// Copyright (c) 2001-2024 by Apryse Software Inc. All Rights Reserved.
//

using System;
using pdftron;
using pdftron.Common;
using pdftron.Filters;
using pdftron.SDF;
using pdftron.PDF;

namespace ElementBuilderTestCS
{
	/// <summary>
	/// Summary description for Class1.
	/// </summary>
	class Class1
	{
		private static pdftron.PDFNetLoader pdfNetLoader = pdftron.PDFNetLoader.Instance();
		static Class1() {}

		// from https://stackoverflow.com/a/228060
		public static string Reverse(string s)
		{
			char[] charArray = s.ToCharArray();
			Array.Reverse(charArray);
			return new string(charArray);
		}

		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main(string[] args)
		{
			PDFNet.Initialize(PDFTronLicense.Key);

			try
			{
				using (PDFDoc pdfdoc = new PDFDoc())
				using (ElementBuilder elementBuilder = new ElementBuilder())
				using (ElementWriter elementWriter = new ElementWriter())
				{
					Page page = pdfdoc.PageCreate(new Rect(0, 0, 612, 794));

					elementWriter.Begin(page);

					// we need to select Indices encoding for shaping
					pdftron.PDF.Font font = pdftron.PDF.Font.CreateCIDTrueTypeFont(pdfdoc, @"c:\Windows\Fonts\ARIALUNI.ttf", true, true, Font.Encoding.e_Indices);

					var element = elementBuilder.CreateTextBegin(font, 32);
					elementWriter.WriteElement(element);

                    ShapedText shapedText = font.GetShapedText(Reverse("مرحبا بالعالم"));
                    ShapedText.FailureReason failureReason = shapedText.GetFailureReason();
                    if (failureReason != ShapedText.FailureReason.e_no_failure)
                    {
                        throw new Exception(failureReason.ToString());
                    }
                    ShapedText.ShapingStatus shapingStatus = shapedText.GetShapingStatus();
                    //Console.WriteLine(shapingStatus);

                    element = elementBuilder.CreateShapedTextRun(shapedText);
                    element.SetTextMatrix(1, 0, 0, 1, 72, 600);
					elementWriter.WriteElement(element);

					shapedText = font.GetShapedText("123");
					failureReason = shapedText.GetFailureReason();
					if (failureReason != ShapedText.FailureReason.e_no_failure)
					{
						throw new Exception(failureReason.ToString());
					}
					shapingStatus = shapedText.GetShapingStatus();
					//Console.WriteLine(shapingStatus);

					element = elementBuilder.CreateShapedTextRun(shapedText);
					element.SetTextMatrix(1, 0, 0, 1, 225, 600);
					elementWriter.WriteElement(element);

                    shapedText = font.GetShapedText(Reverse("اتمنى ان تكون بخير"));
                    failureReason = shapedText.GetFailureReason();
                    if (failureReason != ShapedText.FailureReason.e_no_failure)
                    {
                        throw new Exception(failureReason.ToString());
                    }
                    shapingStatus = shapedText.GetShapingStatus();
                    //Console.WriteLine(shapingStatus);

                    element = elementBuilder.CreateShapedTextRun(shapedText);
                    element.SetTextMatrix(1, 0, 0, 1, 300, 600);
                    elementWriter.WriteElement(element);

                    elementWriter.WriteElement(elementBuilder.CreateTextEnd());

                    elementWriter.End();
					pdfdoc.PagePushBack(page);

					pdfdoc.Save("rtl_ltr_00.pdf", SDFDoc.SaveOptions.e_remove_unused);
				}
			}
			catch (Exception e)
			{
				Console.WriteLine(e);
			}
			PDFNet.Terminate();
		}
	}
}
